@@ -19,6 +19,7 @@
 
 #include <signal.h>
 #include <unistd.h>
+#include <locale.h>
 
 #include <gtk/gtk.h>
 #include <gdk/gdkx.h>
@@ -71,12 +72,26 @@ mt_main_generate_motion_event (GdkScreen *screen, gint x, gint y)
     gdk_error_trap_pop ();
 }
 
+static guint
+mt_main_check_mouse_orientation (MTClosure *mt, guint button)
+{
+    if (mt->left_handed) {
+	if (button == 1)
+	    return 3;
+	else if (button == 3)
+	    return 1;
+    }
+    return button;
+}
+
 static void
 mt_main_generate_button_event (MTClosure *mt,
 			       guint      button,
 			       gint       type,
 			       gulong     delay)
 {
+    button = mt_main_check_mouse_orientation (mt, button);
+
     gdk_error_trap_push ();
     switch (type) {
 	case PRESS:
@@ -328,17 +343,18 @@ dwell_timer_finished (MtTimer *timer, gpointer data)
 static gboolean
 eval_func (Accessible *a, gpointer data)
 {
-    gboolean found;
-    char *name;
+    gchar *name;
+    gboolean found = FALSE;
 
     name = Accessible_getName (a);
-    found = g_str_equal (name, "Window List");
-    SPI_freeString (name);
-
+    if (name) {
+	found = g_str_equal (name, "Window List");
+	SPI_freeString (name);
+    }
     return found;
 }
 
-static gboolean 
+static gboolean
 push_func (Accessible *a, gpointer data)
 {
     MTClosure *mt = data;
@@ -655,6 +671,10 @@ gconf_value_changed (GConfClient *client,
 	else
 	    mt_cursor_manager_restore_all (manager);
     }
+    else if (g_str_equal (key, GNOME_MOUSE_ORIENT) &&
+	     value->type == GCONF_VALUE_BOOL) {
+	mt->left_handed = gconf_value_get_bool (value);
+    }
 }
 
 static void
@@ -683,6 +703,9 @@ get_gconf_options (MTClosure *mt)
 	gconf_client_get_int (mt->client, OPT_G_DRAG, NULL);
     mt->dwell_dirs[DWELL_CLICK_TYPE_RIGHT] =
 	gconf_client_get_int (mt->client, OPT_G_RIGHT, NULL);
+
+    /* mouse orientation */
+    mt->left_handed = gconf_client_get_bool (mt->client, GNOME_MOUSE_ORIENT, NULL);
 }
 
 static void
@@ -758,6 +781,8 @@ mt_closure_init (void)
     }
 
     mt->client = gconf_client_get_default ();
+    gconf_client_add_dir (mt->client, GNOME_MOUSE_DIR,
+			  GCONF_CLIENT_PRELOAD_NONE, NULL);
     gconf_client_add_dir (mt->client, MT_GCONF_HOME,
 			  GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
     g_signal_connect (mt->client, "value_changed",
@@ -820,33 +845,34 @@ main (int argc, char **argv)
     GOptionContext *context;
     GOptionEntry entries[] = {
 	{"enable-dwell", 0, 0, G_OPTION_ARG_NONE, &dwell_click,
-	    _("Enable dwell click"), 0},
+	    N_("Enable dwell click"), 0},
 	{"enable-secondary", 0, 0, G_OPTION_ARG_NONE, &delay_click,
-	    _("Enable simulated secondary click"), 0},
+	    N_("Enable simulated secondary click"), 0},
 	{"dwell-time", 0, 0, G_OPTION_ARG_DOUBLE, &dwell_time,
-	    _("Time to wait before a dwell click"), "[0.2-3.0]"},
+	    N_("Time to wait before a dwell click"), "[0.2-3.0]"},
 	{"secondary-time", 0, 0, G_OPTION_ARG_DOUBLE, &delay_time,
-	    _("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
+	    N_("Time to wait before a simulated secondary click"), "[0.5-3.0]"},
 	{"dwell-mode", 'm', 0, G_OPTION_ARG_STRING, &mode,
-	    _("Dwell mode to use"), "[window|gesture]"},
+	    N_("Dwell mode to use"), "[window|gesture]"},
 	{"show-ctw", 'c', 0, G_OPTION_ARG_NONE, &ctw,
-	    _("Show click type window"), 0},
+	    N_("Show click type window"), 0},
 	{"ctw-x", 'x', 0, G_OPTION_ARG_INT, &pos_x,
-	    _("Window x position"), 0},
+	    N_("Window x position"), 0},
 	{"ctw-y", 'y', 0, G_OPTION_ARG_INT, &pos_y,
-	    _("Window y position"), 0},
+	    N_("Window y position"), 0},
 	{"threshold", 't', 0, G_OPTION_ARG_INT, &threshold,
-	    _("Ignore small pointer movements"), "[0-30]"},
+	    N_("Ignore small pointer movements"), "[0-30]"},
 	{"animate-cursor", 'a', 0, G_OPTION_ARG_NONE, &animate,
-	    _("Show elapsed time as cursor overlay"), 0},
+	    N_("Show elapsed time as cursor overlay"), 0},
 	{"shutdown", 's', 0, G_OPTION_ARG_NONE, &shutdown,
-	    _("Shut down mousetweaks"), 0},
+	    N_("Shut down mousetweaks"), 0},
 	{ NULL }
     };
 
     bindtextdomain (GETTEXT_PACKAGE, LOCALEDIR);
     bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
     textdomain (GETTEXT_PACKAGE);
+    setlocale (LC_ALL, "");
 
     context = g_option_context_new (_("- GNOME mousetweaks daemon"));
     g_option_context_add_main_entries (context, entries, GETTEXT_PACKAGE);
